@@ -479,26 +479,20 @@ static const char * const cmd_send_usage[] = {
 	"which case 'btrfs send' will determine a suitable parent among the",
 	"clone sources itself.",
 	"",
-	"-e               If sending multiple subvols at once, use the new",
-	"                 format and omit the end-cmd between the subvols.",
-	"-p <parent>      Send an incremental stream from <parent> to",
-	"                 <subvol>.",
-	"-c <clone-src>   Use this snapshot as a clone source for an ",
-	"                 incremental send (multiple allowed)",
-	"-f <outfile>     Output is normally written to stdout. To write to",
-	"                 a file, use this option. An alternative would be to",
-	"                 use pipes.",
-	"--no-data        send in NO_FILE_DATA mode, Note: the output stream",
-	"                 does not contain any file data and thus cannot be used",
-	"                 to transfer changes. This mode is faster and useful to",
-	"                 show the differences in metadata.",
-	"--proto N        use protocol version N, or 0 to use the highest version",
-	"                 supported by the sending kernel (default: 1)",
-	"--compressed-data",
-	"                 send data that is compressed on the filesystem directly",
-	"                 without decompressing it",
-	"-v|--verbose     deprecated, alias for global -v option",
-	"-q|--quiet       deprecated, alias for global -q option",
+	OPTLINE("-e", "if sending multiple subvols at once, use the new format and omit the end-cmd between the subvols"),
+	OPTLINE("-p <parent>", "send an incremental stream from <parent> to <subvol>"),
+	OPTLINE("-c <clone-src>", "Use this snapshot as a clone source for an incremental send (multiple allowed)"),
+	OPTLINE("-f <outfile>", "Output is normally written to stdout. To write to "
+		"a file, use this option. An alternative would be to use pipes."),
+	OPTLINE("--no-data", "send in NO_FILE_DATA mode, Note: the output stream "
+		"does not contain any file data and thus cannot be used "
+		"to transfer changes. This mode is faster and useful to "
+		"show the differences in metadata."),
+	OPTLINE("--proto N", "use protocol version N, or 0 to use the highest version "
+		"supported by the sending kernel (default: 1)"),
+	OPTLINE("--compressed-data", "send data that is compressed on the filesystem directly without decompressing it"),
+	OPTLINE("-v|--verbose", "deprecated, alias for global -v option"),
+	OPTLINE("-q|--quiet", "deprecated, alias for global -q option"),
 	HELPINFO_INSERT_GLOBALS,
 	HELPINFO_INSERT_VERBOSE,
 	HELPINFO_INSERT_QUIET,
